package org.firstinspires.ftc.teamcode;

import com.qualcomm.robotcore.eventloop.opmode.OpMode;
import com.qualcomm.robotcore.util.ReadWriteFile;

import org.firstinspires.ftc.robotcore.internal.system.AppUtil;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.File;

public class Replayer extends OpMode {
    private final API.Motor fl = API.Motor.M0;
    private final API.Motor fr = API.Motor.M1;
    private final API.Motor bl = API.Motor.M2;
    private final API.Motor br = API.Motor.M3;

    private final API.Motor intakeMotor = API.Motor.M4;
    private final API.Motor liftMotor = API.Motor.M5;
    private final API.Motor carouselMotor = API.Motor.M6;

    private final String recordingFileName;
    private File recordingFile;
    private JSONObject recordingJSON;
    private int nextIndex = 0;

    public Replayer(String fileName) {
        this.recordingFileName = fileName;
    }

    @Override
    public void init() {
        API.init(this);
        API.print("Status", "Initializing, please wait");
        API.pause(1);

        recordingFile = AppUtil.getInstance().getSettingsFile(recordingFileName);
        try {
            recordingJSON = new JSONObject(ReadWriteFile.readFile(recordingFile));
        } catch (JSONException e) {
            somethingWentWrong(e);
        }

        API.clear();
        API.print("Press play to start");
    }

    @Override
    public void start() {
        API.clear();
        this.resetStartTime();
    }

    @Override
    public void loop() {
        if (nextIndex == -1) return;

        try {
            JSONObject currentListElement = recordingJSON.getJSONArray("recordedData").getJSONObject(nextIndex);
            JSONObject currentData = currentListElement.getJSONObject("data");

            if (this.getRuntime() * 1000 < currentListElement.getDouble("time")) return;

            // Movement
            fl.start(currentData.getDouble("fl"));
            fr.start(currentData.getDouble("fr"));
            bl.start(currentData.getDouble("bl"));
            br.start(currentData.getDouble("br"));

            // Auxiliary
            intakeMotor.start(currentData.getDouble("intake"));
            liftMotor.start(currentData.getDouble("lift"));
            carouselMotor.start(currentData.getDouble("carousel"));

            nextIndex++;

            if (nextIndex >= recordingJSON.getJSONArray("recordedData").length()) {
                this.requestOpModeStop();
                nextIndex = -1;
            }
        } catch (JSONException e) {
            somethingWentWrong(e);
        }
    }

    public void somethingWentWrong(Exception e) {
        API.print(
            "Why are we here? Just to suffer?" + System.lineSeparator() + e.getMessage()
        );
        this.requestOpModeStop();
        nextIndex = -1;
    }
}
